/*---------------------------------------------------------------------------*
  File:     prof_control.h

  Copyright (C) Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#ifndef NN_PROF_PROF_CONTROL_H_
#define NN_PROF_PROF_CONTROL_H_

#include <nn/types.h>
#include <nn/Result.h>

/*! \defgroup control Runtime Control
    \brief These functions are used to control profiling from an application.
*/

/*! \addtogroup control */
/*! \{ */
/*! \defgroup rates Sampling Rates

    These definitions are used to set sampling rates of the profile.
    
    \sa nnprofRuntimeControlSetSampleTime
*/
/*! \{ */
/*! \brief Converts a "times per 60Hz frame" number into nanoseconds.
    \hideinitializer */
#define NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(x)	(1000000000 / (60 * (x)))

/*! \brief Sample at 10 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_10X                NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(10)

/*! \brief Sample at 20 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_20X                NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(20)

/*! \brief Sample at 40 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_40X                NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(40)

/*! \brief Sample at 60 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_60X                NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(60)

/*! \brief Sample at 80 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_80X                NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(80)

/*! \brief Sample at 100 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_100X               NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(100)

/*! \brief Sample at 200 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_200X               NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(200)

/*! \brief Sample at 500 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_500X               NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(500)

/*! \brief Sample at 1000 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_1000X              NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(1000)

/*! \brief Sample at 2000 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_2000X              NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(2000)

/*! \brief Sample at 4000 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_4000X              NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(4000)

/*! \brief Sample at 8000 times per 60Hz frame.
    \hideinitializer */
#define NN_PROF_SETTINGS_8000X              NN_PROF_SETTINGS_SAMPLES_PER_FRAME_TO_NANOSECONDS(8000)
/*! \} */

/*! \brief Used to denote that the currently active thread should be profiled rather than a specific thread ID.

    \sa nnprofRuntimeControlSetThreadIDToProfile

    \ingroup control
*/
#define NN_PROF_SETTINGS_ACTIVE_THREAD_ID   0xFFFFFFFF

/*! \brief Denotes profile options.

    \sa nnprofRuntimeControlSetFlags

    \ingroup control
*/
enum NN_PROF_FLAGS
{
    NN_PROF_FLAGS_NONE                  = 0x00, /*!< No specific flags (simple profiling). */
    NN_PROF_FLAGS_CALLSTACKS            = 0x01, /*!< Choice between individual (simple) samples and callstack samples. */
    NN_PROF_FLAGS_SAMPLEONPERFCOUNTER   = 0x04  /*!< Choice between time and performance counter sampling. */
};

/*! \brief Specifies cores to profile on.

    \sa nnprofRuntimeControlSetCoresToProfile

    \ingroup control
*/
enum NN_PROF_CORE
{
    NN_PROF_CORE_0 = 1, /*!< Core 0: The application core. */
    NN_PROF_CORE_1 = 2, /*!< Core 1: The system core. */

    NN_PROF_CORE_APPLICATION = NN_PROF_CORE_0, /*!< Alias for core 0. */
    NN_PROF_CORE_SYSTEM = NN_PROF_CORE_1       /*!< Alias for core 1. */
};

/*! \brief Specifies a performance counter group to gather data on when profiling.

    The values in this enumeration match those selectable from the PC.

    \sa nnprofRuntimeControlSetPerformanceCounters

    \ingroup control
*/
enum NN_PROF_PERFCOUNTERGROUP
{
    NN_PROF_PERFCOUNTERGROUP_NONE                                   = 0, /*!< Do not record performance counters. */
    NN_PROF_PERFCOUNTERGROUP_INSTRUCTION_MISSES_AND_BUS_CONTENTION  = 1, /*!< Record Instruction Misses and Bus Contention counters. */
    NN_PROF_PERFCOUNTERGROUP_INSTRUCTION_AND_ICACHE_EFFICIENCY      = 2, /*!< Record Instruction and Instruction Cache Efficiency counters. */
    NN_PROF_PERFCOUNTERGROUP_MEMORY_AND_COMPUTE_PERFORMANCE         = 3, /*!< Record Memory and Compute Performance counters. */
    NN_PROF_PERFCOUNTERGROUP_DATA_CACHE_READ_PERFORMANCE            = 4, /*!< Record Data Cache Read Performance counters. */
    NN_PROF_PERFCOUNTERGROUP_DATA_CACHE_WRITE_PERFORMANCE           = 5, /*!< Record Data Cache Write Performance counters. */
    NN_PROF_PERFCOUNTERGROUP_PRESSURE_ON_LOAD_STORE_QUEUE           = 6, /*!< Record Pressure on Load/Store Queue counters. */
    NN_PROF_PERFCOUNTERGROUP_MISPREDICTED_BRANCHES                  = 7, /*!< Record Mispredicted Branches counters. */

    NN_PROF_PERFCOUNTERGROUP_MAX
};

/*! \brief Specifies what kind of sampled profile to take.

    \sa nnprofRuntimeControlSetSampleMethod

    \ingroup control
*/
enum NN_PROF_SAMPLEMETHOD
{
    NN_PROF_SAMPLEMETHOD_TIME                                   = 0, /*!< Sample based on time. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_MISS_ICACHE                = 1, /*!< Sample based on instruction cache misses. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_MISS_DCACHE_READ           = 2, /*!< Sample based on data cache reads. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_MISS_DCACHE_WRITE          = 3, /*!< Sample based on data cache writes. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_BRANCH_MISPREDICTION       = 4, /*!< Sample based on branch mispredictions. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_CYCLES_STALLED_INSTRUCTION = 5, /*!< Sample based on the cycles stalled waiting for an instruction. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_CYCLES_STALLED_DHAZARD     = 6, /*!< Sample based on the cycles stalled due to data hazards. */
    NN_PROF_SAMPLEMETHOD_PERFCOUNTER_CYCLES_STALLED_LSU_FULL    = 7, /*!< Sample based on the cycles stalled due to a full Load/Store queue. */

    NN_PROF_SAMPLEMETHOD_MAX
};
/*! \} */

#if !defined(NN_PROF_DO_NOT_USE)
#ifdef __cplusplus
extern "C" {
#endif

/*! \brief Initializes the Runtime Control library.

    Will set the settings to their default values.
    These will be the same as seen when the PC profiler GUI first starts.
    This function can also be used to easily reset the Runtime Control settings to their default values.

    \ingroup control
*/
void nnprofRuntimeControlInitialize(void);

/*! \brief Sets the flags that will control how profiling occurs.

    \param flags A logical OR of values obtained from the \ref NN_PROF_FLAGS enumeration

    \ingroup control
*/
void nnprofRuntimeControlSetFlags(u32 flags);

/*! \brief Sets what determines when a sample should be taken.

    \param sampleMethod A value from the \ref NN_PROF_SAMPLEMETHOD enumeration.

    \ingroup control
*/
void nnprofRuntimeControlSetSampleMethod(NN_PROF_SAMPLEMETHOD sampleMethod);

/*! \brief Sets the time between samples.

    The value passed into this function will change depending on the sample type (selected in \ref nnprofRuntimeControlSetSampleMethod).

    <i>When sampling by time</i> the <b>time</b> parameter is the time to wait between samples in nanoseconds.
    For ease of use, there are a variety of preset values matching those found in the profiler GUI.
    These take the form \ref rates "NN_PROF_SETTINGS_?X", where ? is the number of times per frame to sample (for example, \ref NN_PROF_SETTINGS_10X).

    <i>When sampling by performance counter</i> the <b>time</b> parameter is the count the performance counter must reach before a sample is taken.
    Preset values based on the GUI will be added a later date.

    \param time The time to wait between samples.

    \ingroup control
*/
void nnprofRuntimeControlSetSampleTime(u32 time);
    
/*! \brief Sets which performance counters to record.

    In the case of profiling by performance counter, this will control which performance counter is used for sampling.

    \param perfGroup A value from the \ref NN_PROF_PERFCOUNTERGROUP enumeration, corresponding to the performance counter groups available in the profiler GUI.

    \ingroup control
*/
void nnprofRuntimeControlSetPerformanceCounters(NN_PROF_PERFCOUNTERGROUP perfGroup);

/*! \brief Sets the thread ID that should be profiled.

    This defaults to the value \ref NN_PROF_SETTINGS_ACTIVE_THREAD_ID, this is equivalent to selecting Active from the profiler GUI settings.
    If an invalid thread is selected, then the profiler will default back to profiling the Active thread.

    A value passed into this function should be the one obtained from nn::os::Thread::GetCurrentId() or nn::os::Thread::GetId().

    \param threadId The ID of the thread to sample.

    \ingroup control
*/
void nnprofRuntimeControlSetThreadIDToProfile(u32 threadId);
  
/*! \brief Sets the core ID that should be profiled.
    
    \deprecated Please use \ref nnprofRuntimeControlSetCoresToProfile instead.

    This defaults to the value NN_PROF_CORE_APPLICATION, this is equivalent to selecting Core 0 from the profiler GUI settings.
    If an invalid core is selected, the profiler will default back to profiling the Core 0.

    \param core The id of the core to sample.

    \ingroup control
*/
void nnprofRuntimeControlSetCoreToProfile(NN_PROF_CORE core);

/*! \brief Sets the core IDs that should be profiled.

    This defaults to the value \ref NN_PROF_CORE_0, this is equivalent to selecting Core 0 from the profiler GUI settings.
    If an invalid core is selected, the profiler will default back to profiling Core 0.

    \param coreMask A logical OR of values obtained from \ref NN_PROF_CORE.

    \ingroup control
*/
void nnprofRuntimeControlSetCoresToProfile(u32 coreMask);

/*! \brief Tells the profiler to start profiling.

    \ingroup control
*/
void nnprofRuntimeControlStart(void);

/*! \brief Tells the profiler to stop profiling.

    \ingroup control
*/
void nnprofRuntimeControlStop(void);

/*! \brief Tells the profiler to enable profiling.

    This is different than starting profiling.
    Calling this function will not start a profile.
    Instead, if a profile is active it will allow the following code to be profiled.
    
    Profiling is enabled by default.

    \ingroup control
*/
void nnprofRuntimeControlEnableProfiling(void);

/*! \brief Tells the profiler to disable profiling.

    This is different than stopping profiling.
    Calling this function will not stop a profile.
    Instead, if a profile is active it will disallow profiling until re-enabled.

    \ingroup control
*/
void nnprofRuntimeControlDisableProfiling(void);

#ifdef __cplusplus
}
#endif
#else
#define nnprofRuntimeControlInitialize()                ((void)0)
#define nnprofRuntimeControlSetFlags(...)				((void)0)
#define nnprofRuntimeControlSetSampleMethod(...)		((void)0)
#define nnprofRuntimeControlSetSampleTime(...)			((void)0)
#define nnprofRuntimeControlSetPerformanceCounters(...)	((void)0)
#define nnprofRuntimeControlSetThreadIDToProfile(...)   ((void)0)
#define nnprofRuntimeControlSetCoreToProfile(...)       ((void)0)
#define nnprofRuntimeControlSetCoresToProfile(...)      ((void)0)
#define nnprofRuntimeControlSetBufferSize(...)			((void)0)
#define nnprofRuntimeControlStart()						((void)0)
#define nnprofRuntimeControlStop()						((void)0)
#define nnprofRuntimeControlEnableProfiling()			((void)0)
#define nnprofRuntimeControlDisableProfiling()			((void)0)
#endif

#endif /* NN_PROF_PROF_CONTROL_H_ */
